#version 150

precision highp float;
precision highp int;

out vec4 FragColor;
struct ShadowControlSt{
	int 	MapWidth;
	int 	MapHeight;
	int 	CubeSize;
	int 	CubeFace;
	float 	SmoothQuality;
	float 	Softness;
} ;

uniform ShadowControlSt ShadowControl;

uniform samplerCube ColorTexture;
in vec3 ecPosition3ToFrag;
void main()
{
	float minFilterSize = 0.0;
	float maxFilterSize = 50.0;
	float FilterSize = minFilterSize + ShadowControl.Softness*(maxFilterSize - minFilterSize);
	int filterSizei = int(FilterSize);
	float PixelSize = 1.0/float(ShadowControl.CubeSize);
	float minJumpSize = 1.0;
	float maxJumpSize = max(FilterSize/2.0,1.0);
	int jump = int(minJumpSize + (1.0-ShadowControl.SmoothQuality)*(maxJumpSize-minJumpSize));
	
	vec2 sumc = vec2(0.0,0.0);
	float sumi = 0.0;
	
	vec3 texCoord;
	vec2 quadCoord = ecPosition3ToFrag.xy;
	vec3 maskI;
	vec3 maskJ;
	
	// get coordinates
	switch (ShadowControl.CubeFace)
	{
	case 0:	// positive X
		texCoord = vec3(1, -quadCoord.y, -quadCoord.x);
		maskI = vec3(0,1,0);
		maskJ = vec3(0,0,1);
		break;
	case 1:	// negative X
		texCoord = vec3(-1, -quadCoord.y, quadCoord.x);
		maskI = vec3(0,1,0);
		maskJ = vec3(0,0,1);
		break;
	case 2:	// positive Y
		texCoord = vec3(quadCoord.x, 1, quadCoord.y);
		maskI = vec3(1,0,0);
		maskJ = vec3(0,0,1);
		break;
	case 3:	// negative Y
		texCoord = vec3(quadCoord.x, -1, -quadCoord.y);
		maskI = vec3(1,0,0);
		maskJ = vec3(0,0,1);
		break;
	case 4:	// positive Z
		texCoord = vec3(quadCoord.x,-quadCoord.y, 1);
		maskI = vec3(0,1,0);
		maskJ = vec3(1,0,0);
		break;
	case 5:	// negative Z
		texCoord = vec3(-quadCoord.x,-quadCoord.y, -1);
		maskI = vec3(0,1,0);
		maskJ = vec3(1,0,0);
		break;
	}

	for (int i=-filterSizei;i<=filterSizei;i=i+jump)
	{
		for (int j=-filterSizei;j<=filterSizei;j=j+jump)
		{
				sumc += texture(ColorTexture, texCoord+float(i)*maskI*PixelSize+float(j)*maskJ*PixelSize).xy;
				sumi+=1.0;
		}
	}
    if (sumi>0.0)
        sumc = sumc / sumi;
    
	FragColor = vec4(sumc,0,1);
}
