#version 150

precision highp float;
precision highp int;


// final color output
out vec4 FragColor;

// input var from vertex shader
in vec4 ProjPxPosToFrag;

// Texture samplers
uniform sampler2D 		FrontPositionMap;
uniform sampler2D 		NormalMap;

const vec2 offs[9] =
    vec2[](
		vec2( 0.0,  0.0), //Center       0
		vec2(-1.0, -1.0), //Top Left     1
		vec2( 1.0,  1.0), //Bottom Right 5
		vec2( 1.0, -1.0), //Top Right    3
		vec2(-1.0,  1.0), //Bottom Left  7
		vec2(-1.0,  0.0),  //Left         8
		vec2( 0.0, -1.0), //Top          2
		vec2(1.0,  0.0),  //Right         8
		vec2( 0.0, 1.0) //Bottom          2	
	);
const vec2 e_barrier = vec2(0.9,0.005);  // x=norm, y=depth
const vec2 e_weights= vec2(1,1);  // x=norm, y=depth

const vec4 avg=vec4(0.25,0.25,0.25,0.25);
const float edge_scale=2.0, edge_distort_scale=3, edge_base=0.0019;
//	0: on edge; 1: internal
float NormalFilter(in vec2 uv, in float width)
{
	// Normal discontinuity filter
	vec3 nc = texture(NormalMap, uv).xyz;
	vec4 nd;
	nd.x = abs(dot(nc, texture(NormalMap, uv + offs[1]* width).xyz));
	nd.y = abs(dot(nc, texture(NormalMap, uv + offs[2]* width).xyz));
	nd.z = abs(dot(nc, texture(NormalMap, uv + offs[3]* width).xyz));
	nd.w = abs(dot(nc, texture(NormalMap, uv + offs[4]* width).xyz));
	nd -= vec4(e_barrier.x);
	nd = step(0, nd);
	
	float ne = clamp(dot(nd, avg)*e_weights.x, 0, 1);
	return ne;
}

//	0: on edge; 1: internal
float DepthFilter(in vec2 uv, in float width)
{
	float dc = texture(FrontPositionMap, uv).r;
	vec4 dd;
	dd.x = texture(FrontPositionMap, uv + offs[1] * width).r + texture(FrontPositionMap, uv + offs[2]* width).r;
	dd.y = texture(FrontPositionMap, uv + offs[3] * width).r + texture(FrontPositionMap, uv + offs[4]* width).r;
	dd.z = texture(FrontPositionMap, uv + offs[5] * width).r + texture(FrontPositionMap, uv + offs[6]* width).r;
	dd.w = texture(FrontPositionMap, uv + offs[7] * width).r + texture(FrontPositionMap, uv + offs[8]* width).r;
		
	dd = abs(2 * dc - dd)- vec4(e_barrier.y);
	dd = step(dd, vec4(0));
	
	
	float de = clamp(dot(dd, avg)*e_weights.y, 0, 1);
	return de;
}

float rand(vec2 co){
    return fract(sin(dot(co ,vec2(12.9898,78.233))) * 43758.5453);
}

vec3 mod289(vec3 x) {
  return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec4 mod289(vec4 x) {
  return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec4 permute(vec4 x) {
     return mod289(((x*34.0)+1.0)*x);
}

vec4 taylorInvSqrt(vec4 r)
{
  return 1.79284291400159 - 0.85373472095314 * r;
}

float snoise(vec3 v)
  { 
  const vec2  C = vec2(1.0/6.0, 1.0/3.0) ;
  const vec4  D = vec4(0.0, 0.5, 1.0, 2.0);

// First corner
  vec3 i  = floor(v + dot(v, C.yyy) );
  vec3 x0 =   v - i + dot(i, C.xxx) ;

// Other corners
  vec3 g = step(x0.yzx, x0.xyz);
  vec3 l = 1.0 - g;
  vec3 i1 = min( g.xyz, l.zxy );
  vec3 i2 = max( g.xyz, l.zxy );

  //   x0 = x0 - 0.0 + 0.0 * C.xxx;
  //   x1 = x0 - i1  + 1.0 * C.xxx;
  //   x2 = x0 - i2  + 2.0 * C.xxx;
  //   x3 = x0 - 1.0 + 3.0 * C.xxx;
  vec3 x1 = x0 - i1 + C.xxx;
  vec3 x2 = x0 - i2 + C.yyy; // 2.0*C.x = 1/3 = C.y
  vec3 x3 = x0 - D.yyy;      // -1.0+3.0*C.x = -0.5 = -D.y

// Permutations
  i = mod289(i); 
  vec4 p = permute( permute( permute( 
             i.z + vec4(0.0, i1.z, i2.z, 1.0 ))
           + i.y + vec4(0.0, i1.y, i2.y, 1.0 )) 
           + i.x + vec4(0.0, i1.x, i2.x, 1.0 ));

// Gradients: 7x7 points over a square, mapped onto an octahedron.
// The ring size 17*17 = 289 is close to a multiple of 49 (49*6 = 294)
  float n_ = 0.142857142857; // 1.0/7.0
  vec3  ns = n_ * D.wyz - D.xzx;

  vec4 j = p - 49.0 * floor(p * ns.z * ns.z);  //  mod(p,7*7)

  vec4 x_ = floor(j * ns.z);
  vec4 y_ = floor(j - 7.0 * x_ );    // mod(j,N)

  vec4 x = x_ *ns.x + ns.yyyy;
  vec4 y = y_ *ns.x + ns.yyyy;
  vec4 h = 1.0 - abs(x) - abs(y);

  vec4 b0 = vec4( x.xy, y.xy );
  vec4 b1 = vec4( x.zw, y.zw );

  //vec4 s0 = vec4(lessThan(b0,0.0))*2.0 - 1.0;
  //vec4 s1 = vec4(lessThan(b1,0.0))*2.0 - 1.0;
  vec4 s0 = floor(b0)*2.0 + 1.0;
  vec4 s1 = floor(b1)*2.0 + 1.0;
  vec4 sh = -step(h, vec4(0.0));

  vec4 a0 = b0.xzyw + s0.xzyw*sh.xxyy ;
  vec4 a1 = b1.xzyw + s1.xzyw*sh.zzww ;

  vec3 p0 = vec3(a0.xy,h.x);
  vec3 p1 = vec3(a0.zw,h.y);
  vec3 p2 = vec3(a1.xy,h.z);
  vec3 p3 = vec3(a1.zw,h.w);

//Normalise gradients
  vec4 norm = taylorInvSqrt(vec4(dot(p0,p0), dot(p1,p1), dot(p2, p2), dot(p3,p3)));
  p0 *= norm.x;
  p1 *= norm.y;
  p2 *= norm.z;
  p3 *= norm.w;

// Mix final noise value
  vec4 m = max(0.6 - vec4(dot(x0,x0), dot(x1,x1), dot(x2,x2), dot(x3,x3)), 0.0);
  m = m * m;
  return 42.0 * dot( m*m, vec4( dot(p0,x0), dot(p1,x1), 
                                dot(p2,x2), dot(p3,x3) ) );
}

  float snoise(vec2 v)
  {
	return snoise(vec3(v,0));
  }

void main()
{
	float edge_width=edge_base*edge_scale;
	vec2 uv=0.5*ProjPxPosToFrag.xy/ProjPxPosToFrag.w+0.5;
	float hatch0=snoise(20*uv);
	float hatch1=snoise(20-20*uv);
	vec2 hatch=vec2(hatch0, hatch1);
	hatch=hatch*edge_base*edge_distort_scale;
	float normalFilt=NormalFilter(uv+hatch, edge_width);
	float depthFilt=DepthFilter(uv+hatch, edge_width);
	float w = normalFilt*depthFilt;
	
	w=clamp(w*1.5, 0, 1);
		FragColor = vec4(w, w, w, 1.0);
}
