#version 150

precision highp float;
precision highp int;


layout (lines) in;
layout (line_strip, max_vertices = 36) out;

smooth out vec4 pos;
out vec4 source;

uniform float LineWidth;
uniform int  RenderArea_Width;
uniform int  RenderArea_Height;

void main(void)
{
    vec4 pos1 = gl_in[0].gl_Position;
    vec4 pos2 = gl_in[1].gl_Position;

    if (LineWidth <= 1.0)
    {
        //  draw the single center line
        gl_Position = pos1;
        pos = pos1;
        source = pos1;
        EmitVertex();
        gl_Position = pos2;
        pos = pos2;
        source = pos1;
        EmitVertex();
        EndPrimitive();
    }
    else
    {
        int i;
        int lineWidth2;

        vec3 pos1XYZ = pos1.xyz/pos1.w;
        vec3 pos2XYZ = pos2.xyz/pos2.w;

        vec2 pos1XY = pos1XYZ.xy;
        vec2 pos2XY = pos2XYZ.xy;

        vec2 dirXY = pos2XY-pos1XY;

        if (abs(dirXY.x) > 1e-8 || abs(dirXY.y) > 1e-8)
        {
            dirXY = normalize(dirXY);
            
            vec2 normalXY = vec2(dirXY.y, -dirXY.x);
            vec2 extend = normalXY*1;
            extend = vec2(extend.x/float(RenderArea_Width), extend.y/float(RenderArea_Height));
            lineWidth2 = int(round(LineWidth));
            
            //  draw the center line
            gl_Position = pos1;
            pos = pos1;
            source = pos1;
            EmitVertex();
            gl_Position = pos2;
            pos = pos2;
            source = pos1;
            EmitVertex();
            EndPrimitive();
            
            //  draw the side lines to make it look bold
            for (i = 1; i < lineWidth2; i++)
            {
                gl_Position = vec4((pos1XY-extend*i)*pos1.w, pos1.z, pos1.w);
                pos = pos1;
                source = pos1;
                EmitVertex();
                gl_Position = vec4((pos2XY-extend*i)*pos2.w, pos2.z, pos2.w);
                pos = pos2;
                source = pos1;
                EmitVertex();
                EndPrimitive();
            
                gl_Position = vec4((pos1XY+extend*i)*pos1.w, pos1.z, pos1.w);
                pos = pos1;
                source = pos1;
                EmitVertex();
                gl_Position = vec4((pos2XY+extend*i)*pos2.w, pos2.z, pos2.w);
                pos = pos2;
                source = pos1;
                EmitVertex();
                EndPrimitive();
            }
        }
    }
}
