#version 150

precision highp float;
precision highp int;

#define LIGHT_TYPE_AMBIENT		0
#define LIGHT_TYPE_POINT		1
#define LIGHT_TYPE_DISTANT		2
#define	LIGHT_TYPE_CONE			3

// vertex attributes
in vec3 MCVertex;
in vec3 MCNormal;
in vec2 MCTexCoord;
in vec4 MCColor;
in vec3 MCTangent;
// out var to fragment shader
out vec3 MCVertexToFrag;
out vec3 NormalToFrag;
out vec4 PxPosToFrag;
out vec4 ProjPxPosToFrag;
//out vec4 LightPosToFrag;
//out vec4 LightDirToFrag;
//out vec4 LightToPxToFrag;
out vec4 PxPosinLightsView;
out vec4 ShadowMapTexCoord;
out float depthLinear;
out vec2 TexCoordToFrag;
out vec4 TangentToFrag;
out vec4 ColorToFrag;
// transformation matrices
//uniform mat4 renderPJMatrix;
//uniform mat4 renderVMMatrix;
//uniform mat4 renderMMMatrix;
//uniform mat3 renderNMMatrix;
//uniform mat4 lightPJMatrix, lightVMMatrix;
//MJY 06/23/14
uniform mat4 lightPJMatrix;
uniform mat4 lightmvMatrix;
uniform mat4 renderPJMatrix;
uniform mat4 rendermvMatrix;
uniform mat3 renderNMMatrix;

// light
struct LightSt{
	int 	Type;
	// generic
	vec3 	Color;
	float	Factor;
	int 	Falloff;
	int		ShadowOn;
	// location
	vec3 	Origin;
	vec3 	Direction;
	// point light
	float 	Radius;
	// cone light
	float	InAngle;
	float	OutAngle;
	// view
	float 	NearClip;
	float 	FarClip;
	// control
	int		InEyeSpace;
} ;

uniform LightSt Light;

uniform int Light_Num;

struct MaterialSt{
	vec3 	Ambient;
	vec3 	Diffuse;
	vec3 	Specular;
	float	Shininess;
	int 	Has_Specular;
	int 	Has_Texture;
    int     Do_Sphere_Map;
    int     Has_Vertex_Color;
	int 	Has_Bump;
    int     Receive_Light;
	int 	Is_Transparent;
    float   Transparency;
	int 	Receive_Shadow;
	int		Cast_Shadow;
	int		Use_AO;
    
    int     Const_Refl;     /*constant reflection on/off*/
} ;

uniform MaterialSt Material;

struct ShadowControlSt {
    int 	MapWidth;
	int 	MapHeight;
	int 	CubeSize;
	int 	CubeFace;
	float 	SmoothQuality;
	float 	Softness;
	float   Darkness;
	int     ShadowOnGlobal;
};

uniform ShadowControlSt ShadowControl;

struct AOControlSt{
	int	  IsEnabled;
};

uniform AOControlSt AOControl;


void main()
{
	MCVertexToFrag=MCVertex;

	NormalToFrag = normalize(renderNMMatrix * MCNormal);
	//mat4 renderMV = renderVMMatrix * renderMMMatrix;
	TangentToFrag = normalize(rendermvMatrix*vec4(MCTangent,0.0));
	
	ColorToFrag = MCColor;

	vec4 pos = rendermvMatrix * vec4(MCVertex,1);
	
    /*MJY 07/16/13 : sphere map coordinates for analyzed objects*/
    if (Material.Do_Sphere_Map > 0)
    {
        vec3 u = normalize( vec3(pos) );
        vec3 n = NormalToFrag;
        vec3 r = reflect( u, n );
        float m = 2.0 * sqrt( r.x*r.x + r.y*r.y + (r.z+1.0)*(r.z+1.0) );
		
		TexCoordToFrag=r.xy/m+0.5;			
    }
    else
    {
        TexCoordToFrag = MCTexCoord;
    }
    
    
	if (Light.Type!=LIGHT_TYPE_AMBIENT)
	{
		PxPosToFrag = pos;
		
		//LightPosToFrag=(Light.InEyeSpace>0)?vec4(Light.Origin,1.0):(renderVMMatrix*vec4(Light.Origin,1.0));
		//LightDirToFrag=(Light.InEyeSpace>0)?vec4(Light.Direction,0.0):(renderVMMatrix*vec4(Light.Direction,0.0));
		
		
		// for shadow
		if (ShadowControl.ShadowOnGlobal>0 && Light.ShadowOn>0)
		{
			//PxPosinLightsView = lightVMMatrix * renderMMMatrix * vec4(MCVertex,1);
			PxPosinLightsView = lightmvMatrix * vec4(MCVertex,1);
		
			if (Light.Type==LIGHT_TYPE_DISTANT || Light.Type==LIGHT_TYPE_CONE)
			{
				mat4 M_bias = mat4(0.5,0,0,0,  0,0.5,0,0,  0,0,0.5,0,  0.5,0.5,0.5,1);      //  [-1, 1] to [0, 1]
				//mat4 M = M_bias * lightPJMatrix * lightVMMatrix;
				//ShadowMapTexCoord = M * renderMMMatrix * vec4(MCVertex,1);
				mat4 M = M_bias * lightPJMatrix * lightmvMatrix;
				ShadowMapTexCoord = M * vec4(MCVertex,1);
			}
			if (Light.Type==LIGHT_TYPE_DISTANT || Light.Type==LIGHT_TYPE_CONE)
			{
				depthLinear = (-PxPosinLightsView.z/PxPosinLightsView.w - Light.NearClip) / (Light.FarClip - Light.NearClip);
			}
		}
	}
	
	ProjPxPosToFrag = renderPJMatrix * pos;
	gl_Position = ProjPxPosToFrag;

	
}
