#version 150

precision highp float;
precision highp int;

#define LIGHT_TYPE_AMBIENT		0
#define LIGHT_TYPE_POINT		1
#define LIGHT_TYPE_DISTANT		2
#define	LIGHT_TYPE_CONE			3
#define ALPHA_THRES             0.009   /*above which the pixel is rendered*/
// final color output
out vec4 FragColor;
// input var from vertex shader
in vec4 PxPos;
in vec4 ColorToFrag;
in vec2 TexCoordToFrag;

// light
struct LightSt{
	int 	Type;
	float 	NearClip;
	float 	FarClip;
} ;

uniform LightSt Light;

struct MaterialSt {
	int 	Has_Texture;
    int     Do_Sphere_Map;
    int     Has_Vertex_Color;
	int 	Is_Transparent;
    float   Transparency;
	int		Cast_Shadow;
} ;

uniform MaterialSt Material;

// Texture samplers
uniform sampler2D 		ColorTexture;

void main()
{
    bool cast_shadow=true;
    
    if (Material.Cast_Shadow==0)
        cast_shadow=false;
    
    if (cast_shadow)
    {
        if (Material.Has_Texture>0)
        {
            vec4 tex_Color = texture(ColorTexture, TexCoordToFrag);
            cast_shadow=((Material.Is_Transparent>0?Material.Transparency:1.0) * (Material.Do_Sphere_Map>0?1.0:tex_Color.a))>ALPHA_THRES;
        }
        else if (Material.Has_Vertex_Color>0)
        {
             cast_shadow=((Material.Is_Transparent>0?Material.Transparency:1.0) * ColorToFrag.a)>ALPHA_THRES;
        }
    else
            cast_shadow=(Material.Is_Transparent>0?Material.Transparency:1.0)>ALPHA_THRES;
    }
    
    
    
    if (cast_shadow)
    {
        float depth;
        if (Light.Type == LIGHT_TYPE_POINT)
        {
            // distance instead of depth
            depth = (length(PxPos.xyz/PxPos.w)-Light.NearClip)/(1.414*Light.FarClip-Light.NearClip);
        }
        else if (Light.Type == LIGHT_TYPE_DISTANT || Light.Type == LIGHT_TYPE_CONE)
        {
            depth = (-PxPos.z/PxPos.w - Light.NearClip) / (Light.FarClip - Light.NearClip);
        }
        
		
		//	MJY 06/09/14: Variance shadow map creation
		float moment1 = depth;
		float moment2 = depth * depth;
		// Adjusting moments (this is sort of bias per pixel) using derivative
		float dx = dFdx(depth);
		float dy = dFdy(depth);
		moment2 += 0.25*(dx*dx+dy*dy) ;
		FragColor = vec4( moment1,moment2, 0.0, 0.0 );		
        //FragColor = vec4(depth, 0,0,1); // the rest 3 slots reserved for other uses
    }
    else
        discard;
}
